@@ -1,4 +1,4 @@
-import sys, os, pathlib, warnings, datetime, tempfile, glob
+import sys, os, pathlib, warnings, datetime, tempfile, glob, time
 import gc
 from natsort import natsorted
 from tqdm import tqdm
@@ -6,19 +6,22 @@ from tqdm import tqdm
 from PyQt5 import QtGui, QtCore, Qt, QtWidgets
 import pyqtgraph as pg
 from pyqtgraph import GraphicsScene
-import matplotlib.pyplot as plt
 
 import numpy as np
 import cv2
 from scipy.ndimage import gaussian_filter
-from skimage import io
-from skimage import draw
 
 import mxnet as mx
 from mxnet import nd
 
 from . import utils, transforms, models, guiparts, plot, menus, io, dynamics
 
+try:
+    import matplotlib.pyplot as plt
+    MATPLOTLIB = True
+except:
+    MATPLOTLIB = False
+
 try:
     from google.cloud import storage
     os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = os.path.join(os.path.dirname(os.path.realpath(__file__)),
@@ -101,9 +104,11 @@ def run(image=None):
     # Always start by initializing Qt (only once per application)
     warnings.filterwarnings("ignore")
     app = QtGui.QApplication(sys.argv)
-    icon_path = os.path.join(
-        os.path.dirname(os.path.realpath(__file__)), 'logo/logo.png'
-    )
+    icon_path = pathlib.Path.home().joinpath('.cellpose', 'logo.png')
+    if not icon_path.is_file():
+        print('downloading logo')
+        utils.download_url_to_file('http://www.cellpose.org/static/images/cellpose_transparent.png', icon_path, progress=True)
+    icon_path = str(icon_path.resolve())
     app_icon = QtGui.QIcon()
     app_icon.addFile(icon_path, QtCore.QSize(16, 16))
     app_icon.addFile(icon_path, QtCore.QSize(24, 24))
@@ -135,9 +140,8 @@ class MainW(QtGui.QMainWindow):
         self.setWindowTitle("cellpose")
         self.cp_path = os.path.dirname(os.path.realpath(__file__))
         app_icon = QtGui.QIcon()
-        icon_path = os.path.abspath(os.path.join(
-            self.cp_path, "logo/logo.png")
-        )
+        icon_path = pathlib.Path.home().joinpath('.cellpose', 'logo.png')
+        icon_path = str(icon_path.resolve())
         app_icon.addFile(icon_path, QtCore.QSize(16, 16))
         app_icon.addFile(icon_path, QtCore.QSize(24, 24))
         app_icon.addFile(icon_path, QtCore.QSize(32, 32))
@@ -170,7 +174,7 @@ class MainW(QtGui.QMainWindow):
         self.l0 = QtGui.QGridLayout()
         self.cwidget.setLayout(self.l0)
         self.setCentralWidget(self.cwidget)
-        self.l0.setVerticalSpacing(4)
+        self.l0.setVerticalSpacing(6)
 
         self.imask = 0
 
@@ -188,7 +192,10 @@ class MainW(QtGui.QMainWindow):
         for i in range(3):
             self.cmap.append(make_cmap(i).getLookupTable(start=0.0, stop=255.0, alpha=False))
 
-        self.colormap = (plt.get_cmap('gist_ncar')(np.linspace(0.0,.9,1000)) * 255).astype(np.uint8)
+        if MATPLOTLIB:
+            self.colormap = (plt.get_cmap('gist_ncar')(np.linspace(0.0,.9,1000)) * 255).astype(np.uint8)
+        else:
+            self.colormap = ((np.random.rand(1000,3)*0.8+0.1)*255).astype(np.uint8)
         self.reset()
 
         self.is_stack = True # always loading images of same FOV
@@ -206,7 +213,16 @@ class MainW(QtGui.QMainWindow):
         HW.show()
 
     def make_buttons(self):
-        self.boldfont = QtGui.QFont("Arial", 10, QtGui.QFont.Bold)
+        label_style = """QLabel{
+                            color: white
+                            } 
+                         QToolTip { 
+                           background-color: black; 
+                           color: white; 
+                           border: black solid 1px
+                           }"""
+        self.boldfont = QtGui.QFont("Arial", 12, QtGui.QFont.Bold)
+        self.medfont = QtGui.QFont("Arial", 10)
         self.smallfont = QtGui.QFont("Arial", 8)
         self.headings = ('color: rgb(150,255,150);')
         self.dropdowns = ("color: white;"
@@ -220,13 +236,13 @@ class MainW(QtGui.QMainWindow):
         label.setFont(self.boldfont)
         self.l0.addWidget(label, 0,0,1,1)
 
-        label = QtGui.QLabel('[W/S]')
-        label.setStyleSheet('color: white')
-        #label.setFont(self.smallfont)
+        label = QtGui.QLabel('[up/down or W/S]')
+        label.setStyleSheet(label_style)
+        label.setFont(self.smallfont)
         self.l0.addWidget(label, 1,0,1,1)
 
         label = QtGui.QLabel('[pageup/down]')
-        label.setStyleSheet('color: white')
+        label.setStyleSheet(label_style)
         label.setFont(self.smallfont)
         self.l0.addWidget(label, 1,1,1,1)
 
@@ -236,6 +252,7 @@ class MainW(QtGui.QMainWindow):
         self.RGBChoose = guiparts.RGBRadioButtons(self, b,1)
         self.RGBDropDown = QtGui.QComboBox()
         self.RGBDropDown.addItems(["RGB","gray","red","green","blue"])
+        self.RGBDropDown.setFont(self.medfont)
         self.RGBDropDown.currentIndexChanged.connect(self.color_choose)
         self.RGBDropDown.setFixedWidth(60)
         self.RGBDropDown.setStyleSheet(self.dropdowns)
@@ -263,9 +280,11 @@ class MainW(QtGui.QMainWindow):
         self.BrushChoose.currentIndexChanged.connect(self.brush_choose)
         self.BrushChoose.setFixedWidth(60)
         self.BrushChoose.setStyleSheet(self.dropdowns)
+        self.BrushChoose.setFont(self.medfont)
         self.l0.addWidget(self.BrushChoose, b, 1,1,1)
         label = QtGui.QLabel('brush size: [, .]')
-        label.setStyleSheet('color: white;')
+        label.setStyleSheet(label_style)
+        label.setFont(self.medfont)
         self.l0.addWidget(label, b,0,1,1)
 
         # cross-hair
@@ -276,6 +295,7 @@ class MainW(QtGui.QMainWindow):
         # turn on draw mode
         self.SCheckBox = QtGui.QCheckBox('single stroke')
         self.SCheckBox.setStyleSheet(self.checkstyle)
+        self.SCheckBox.setFont(self.medfont)
         self.SCheckBox.toggled.connect(self.autosave_on)
         self.l0.addWidget(self.SCheckBox, b,0,1,2)
 
@@ -283,6 +303,7 @@ class MainW(QtGui.QMainWindow):
         # turn on crosshairs
         self.CHCheckBox = QtGui.QCheckBox('cross-hairs')
         self.CHCheckBox.setStyleSheet(self.checkstyle)
+        self.CHCheckBox.setFont(self.medfont)
         self.CHCheckBox.toggled.connect(self.cross_hairs)
         self.l0.addWidget(self.CHCheckBox, b,0,1,1)
 
@@ -292,6 +313,7 @@ class MainW(QtGui.QMainWindow):
         self.masksOn = True
         self.MCheckBox = QtGui.QCheckBox('MASKS ON [X]')
         self.MCheckBox.setStyleSheet(self.checkstyle)
+        self.MCheckBox.setFont(self.medfont)
         self.MCheckBox.setChecked(True)
         self.MCheckBox.toggled.connect(self.toggle_masks)
         self.l0.addWidget(self.MCheckBox, b,0,1,2)
@@ -301,6 +323,7 @@ class MainW(QtGui.QMainWindow):
         self.outlinesOn = True
         self.OCheckBox = QtGui.QCheckBox('outlines on [Z]')
         self.OCheckBox.setStyleSheet(self.checkstyle)
+        self.OCheckBox.setFont(self.medfont)
         self.OCheckBox.setChecked(True)
         self.OCheckBox.toggled.connect(self.toggle_masks)
         self.l0.addWidget(self.OCheckBox, b,0,1,2)
@@ -326,11 +349,14 @@ class MainW(QtGui.QMainWindow):
 
         b+=1
         self.diameter = 30
-        label = QtGui.QLabel('cell diameter (pix):')
-        label.setStyleSheet('color: white;')
+        label = QtGui.QLabel('cell diameter (pixels) (click ENTER):')
+        label.setStyleSheet(label_style)
+        label.setFont(self.medfont)
+        label.setToolTip('you can manually enter the approximate diameter for your cells, or press “calibrate” to let the model estimate it. The size is represented by a disk at the bottom of the view window (can turn this disk of by unchecking “scale disk on”)')
         self.l0.addWidget(label, b, 0,1,2)
         self.Diameter = QtGui.QLineEdit()
         self.Diameter.setText(str(self.diameter))
+        self.Diameter.setFont(self.medfont)
         self.Diameter.returnPressed.connect(self.compute_scale)
         self.Diameter.setFixedWidth(50)
         b+=1
@@ -348,8 +374,10 @@ class MainW(QtGui.QMainWindow):
         b+=1
         self.scale_on = True
         self.ScaleOn = QtGui.QCheckBox('scale disk on')
+        self.ScaleOn.setFont(self.medfont)
         self.ScaleOn.setStyleSheet('color: red;')
         self.ScaleOn.setChecked(True)
+        self.ScaleOn.setToolTip('see current diameter as red disk at bottom')
         self.ScaleOn.toggled.connect(self.toggle_scale)
         self.l0.addWidget(self.ScaleOn, b,0,1,2)
 
@@ -357,8 +385,18 @@ class MainW(QtGui.QMainWindow):
         b+=1
         self.useGPU = QtGui.QCheckBox('use GPU')
         self.useGPU.setStyleSheet(self.checkstyle)
+        self.useGPU.setFont(self.medfont)
+        self.useGPU.setToolTip('if you have specially installed the <i>cuda</i> version of mxnet, then you can activate this, but it won’t give huge speedups when running single 2D images in the GUI.')
         self.check_gpu()
-        self.l0.addWidget(self.useGPU, b,0,1,2)
+        self.l0.addWidget(self.useGPU, b,0,1,1)
+
+        # fast mode
+        self.FastMode = QtGui.QCheckBox('fast mode')
+        self.FastMode.setStyleSheet(self.checkstyle)
+        self.FastMode.setFont(self.medfont)
+        self.FastMode.setChecked(False)
+        self.FastMode.setToolTip('turn off augmentations + averaging 4 different fit networks to <i>increase</i> run speed')
+        self.l0.addWidget(self.FastMode, b,1,1,1)
 
         b+=1
         # choose models
@@ -368,9 +406,12 @@ class MainW(QtGui.QMainWindow):
         self.ModelChoose.addItems(models)
         self.ModelChoose.setFixedWidth(70)
         self.ModelChoose.setStyleSheet(self.dropdowns)
+        self.ModelChoose.setFont(self.medfont)
         self.l0.addWidget(self.ModelChoose, b, 1,1,1)
         label = QtGui.QLabel('model: ')
-        label.setStyleSheet('color: white;')
+        label.setStyleSheet(label_style)
+        label.setFont(self.medfont)
+        label.setToolTip('there is a <em>cytoplasm</em> model and a <em>nuclei</em> model, choose what you want to segment')
         self.l0.addWidget(label, b, 0,1,1)
 
         b+=1
@@ -378,12 +419,18 @@ class MainW(QtGui.QMainWindow):
         self.ChannelChoose = [QtGui.QComboBox(), QtGui.QComboBox()]
         self.ChannelChoose[0].addItems(['gray','red','green','blue'])
         self.ChannelChoose[1].addItems(['none','red','green','blue'])
-        cstr = ['chan to seg', 'chan2 (opt)']
+        cstr = ['chan to segment:', 'chan2 (optional): ']
         for i in range(2):
             self.ChannelChoose[i].setFixedWidth(70)
             self.ChannelChoose[i].setStyleSheet(self.dropdowns)
+            self.ChannelChoose[i].setFont(self.medfont)
             label = QtGui.QLabel(cstr[i])
-            label.setStyleSheet('color: white;')
+            label.setStyleSheet(label_style)
+            label.setFont(self.medfont)
+            if i==0:
+                label.setToolTip('this is the channel in which the cytoplasm or nuclei exist that you want to segment')
+            else:
+                label.setToolTip('if <em>cytoplasm</em> model is chosen, and you also have a nuclear channel, then choose the nuclear channel for this option')
             self.l0.addWidget(label, b, 0,1,1)
             self.l0.addWidget(self.ChannelChoose[i], b, 1,1,1)
             b+=1
@@ -392,6 +439,7 @@ class MainW(QtGui.QMainWindow):
         b+=1
         self.invert = QtGui.QCheckBox('invert grayscale')
         self.invert.setStyleSheet(self.checkstyle)
+        self.invert.setFont(self.medfont)
         self.l0.addWidget(self.invert, b,0,1,2)
 
         b+=1
@@ -412,7 +460,8 @@ class MainW(QtGui.QMainWindow):
         b+=1
         label = QtGui.QLabel('flow error threshold:')
         label.setToolTip('threshold on flow error to accept for masks (set higher to get more cells)')
-        label.setStyleSheet('color: white;')
+        label.setStyleSheet(label_style)
+        label.setFont(self.medfont)
         self.l0.addWidget(label, b, 0,1,2)
 
         b+=1
@@ -424,11 +473,13 @@ class MainW(QtGui.QMainWindow):
         self.threshslider.setValue(4)
         self.l0.addWidget(self.threshslider, b, 0,1,2)
         self.threshslider.valueChanged.connect(self.compute_cprob)
-        #self.threshslider.setEnabled(False)
+        self.threshslider.setStyleSheet(guiparts.horizontal_slider_style())
+        self.threshslider.setEnabled(False)
         
         b+=1
         label = QtGui.QLabel('cell prob threshold:')
-        label.setStyleSheet('color: white;')
+        label.setStyleSheet(label_style)
+        label.setFont(self.medfont)
         self.l0.addWidget(label, b, 0,1,2)
         label.setToolTip('cell probability threshold (set lower to get more cells)')
         
@@ -441,8 +492,8 @@ class MainW(QtGui.QMainWindow):
         self.cellprob = 0.0
         self.l0.addWidget(self.probslider, b, 0,1,2)
         self.probslider.valueChanged.connect(self.compute_cprob)
-        #self.probslider.setEnabled(False)
-
+        self.probslider.setStyleSheet(guiparts.horizontal_slider_style())
+        self.probslider.setEnabled(False)
 
         b+=1
         line = QHLine()
@@ -451,6 +502,7 @@ class MainW(QtGui.QMainWindow):
 
         self.autobtn = QtGui.QCheckBox('auto-adjust')
         self.autobtn.setStyleSheet(self.checkstyle)
+        self.autobtn.setFont(self.medfont)
         self.autobtn.setChecked(True)
         self.l0.addWidget(self.autobtn, b+2,0,1,1)
 
@@ -469,13 +521,13 @@ class MainW(QtGui.QMainWindow):
         self.slider.setTickPosition(QtGui.QSlider.TicksRight)
         self.l0.addWidget(self.slider, b,1,1,1)
         self.l0.setRowStretch(b, 1)
-
+        
         b+=2
         # add z position underneath
         self.currentZ = 0
         label = QtGui.QLabel('Z:')
         label.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
-        label.setStyleSheet('color: white;')
+        label.setStyleSheet(label_style)
         self.l0.addWidget(label, b, 0,1,1)
         self.zpos = QtGui.QLineEdit()
         self.zpos.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
@@ -573,6 +625,8 @@ class MainW(QtGui.QMainWindow):
                         self.undo_action()
                     if event.key() == QtCore.Qt.Key_0:
                         self.clear_all()
+        if event.key() == QtCore.Qt.Key_Minus or event.key() == QtCore.Qt.Key_Equal:
+            self.p0.keyPressEvent(event)
 
     def check_gpu(self):
         if utils.use_gpu():
@@ -630,7 +684,7 @@ class MainW(QtGui.QMainWindow):
         else:
             # remove previous cell
             if self.ncells> 0:
-                self.remove_cell(self.ncells-1)
+                self.remove_cell(self.ncells)
 
     def get_files(self):
         images = []
@@ -749,7 +803,7 @@ class MainW(QtGui.QMainWindow):
         if self.autobtn.isChecked():
             self.saturation = [[0,255] for n in range(self.NZ)]
         self.currentZ = 0
-        self.flows = [[],[],[]]
+        self.flows = [[],[],[],[],[[]]]
         self.stack = np.zeros((1,self.Ly,self.Lx,3))
         # masks matrix
         self.layers = 0*np.ones((1,self.Ly,self.Lx,4), np.uint8)
@@ -919,7 +973,7 @@ class MainW(QtGui.QMainWindow):
             image = np.zeros((self.Ly,self.Lx), np.uint8)
             if len(self.flows)>=self.view-1 and len(self.flows[self.view-1])>0:
                 image = self.flows[self.view-1][self.currentZ]
-            if self.view>2:
+            if self.view>1:
                 self.img.setImage(image, autoLevels=False, lut=self.bwr)
             else:
                 self.img.setImage(image, autoLevels=False, lut=None)
@@ -929,8 +983,8 @@ class MainW(QtGui.QMainWindow):
         #self.img.set_ColorMap(self.bwr)
         if self.masksOn or self.outlinesOn:
             self.layer.setImage(self.layers[self.currentZ], autoLevels=False)
-        self.slider.setLow(self.saturation[self.currentZ][0])
-        self.slider.setHigh(self.saturation[self.currentZ][1])
+        #self.slider.setLow(self.saturation[self.currentZ][0])
+        #self.slider.setHigh(self.saturation[self.currentZ][1])
         self.win.show()
         self.show()
 
@@ -972,9 +1026,17 @@ class MainW(QtGui.QMainWindow):
             iz = points[:,0] == z
             vr = points[iz,1]
             vc = points[iz,2]
-
-            vr, vc = draw.polygon_perimeter(vr, vc, self.layers[z].shape[:2])
-            ar, ac = draw.polygon(vr, vc, self.layers[z].shape[:2])
+            # get points inside drawn points
+            mask = np.zeros((np.ptp(vr)+4, np.ptp(vc)+4), np.uint8)
+            pts = np.stack((vc-vc.min()+2,vr-vr.min()+2), axis=-1)[:,np.newaxis,:]
+            mask = cv2.fillPoly(mask, [pts], (255,0,0))
+            ar, ac = np.nonzero(mask)
+            ar, ac = ar+vr.min()-2, ac+vc.min()-2
+            # get dense outline
+            contours = cv2.findContours(mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_NONE)
+            pvc, pvr = contours[0][0].squeeze().T            
+            vr, vc = pvr + vr.min() - 2, pvc + vc.min() - 2
+            # concatenate all points
             ar, ac = np.hstack((np.vstack((vr, vc)), np.vstack((ar, ac))))
             # if these pixels are overlapping with another cell, reassign them
             ioverlap = self.cellpix[z][ar, ac] > 0
@@ -986,10 +1048,9 @@ class MainW(QtGui.QMainWindow):
                 # compute outline of new mask
                 mask = np.zeros((np.ptp(ar)+4, np.ptp(ac)+4), np.uint8)
                 mask[ar-ar.min()+2, ac-ac.min()+2] = 1
-                outlines = plot.masks_to_outlines(mask)
-                vr, vc = np.nonzero(outlines)
-                vr, vc = vr + ar.min() - 2, vc + ac.min() - 2
-
+                contours = cv2.findContours(mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_NONE)
+                pvc, pvr = contours[0][0].squeeze().T            
+                vr, vc = pvr + ar.min() - 2, pvc + ac.min() - 2
             self.draw_mask(z, ar, ac, vr, vc, color)
 
             median.append(np.array([np.median(ar), np.median(ac)]))
@@ -1144,6 +1205,7 @@ class MainW(QtGui.QMainWindow):
     def compute_model(self):
         self.progress.setValue(0)
         if 1:
+            tic=time.time()
             self.clear_all()
             self.flows = [[],[],[]]
             self.initialize_model()
@@ -1159,8 +1221,10 @@ class MainW(QtGui.QMainWindow):
             channels = self.get_channels()
             self.diameter = float(self.Diameter.text())
             try:
+                augment = (not self.FastMode.isChecked())
                 masks, flows, _, _ = self.model.eval(data, channels=channels,
                                                 diameter=self.diameter, invert=self.invert.isChecked(),
+                                                net_avg=augment, augment=augment,
                                                 do_3D=do_3D, progress=self.progress)
             except Exception as e:
                 print('NET ERROR: %s'%e)
@@ -1184,9 +1248,8 @@ class MainW(QtGui.QMainWindow):
             if len(flows)>2:
                 self.flows.append(flows[3])
                 self.flows.append(np.concatenate((flows[1], flows[2][np.newaxis,...]), axis=0))
-                print(self.flows[3].shape, self.flows[4].shape)
-
-            print('%d cells found with cellpose net'%(len(np.unique(masks)[1:])))
+                
+            print('%d cells found with cellpose net in %0.3f sec'%(len(np.unique(masks)[1:]), time.time()-tic))
             self.progress.setValue(80)
             z=0
             self.masksOn = True
@@ -1198,7 +1261,10 @@ class MainW(QtGui.QMainWindow):
             self.progress.setValue(100)
 
             self.toggle_server(off=True)
-        else:#except Exception as e:
+            if not do_3D:
+                self.threshslider.setEnabled(True)
+                self.probslider.setEnabled(True)
+        else: #except Exception as e:
             print('ERROR: %s'%e)
 
 
@@ -1212,6 +1278,7 @@ class MainW(QtGui.QMainWindow):
         self.loadMasks.setEnabled(True)
         self.saveSet.setEnabled(True)
         self.savePNG.setEnabled(True)
+        self.saveOutlines.setEnabled(True)
         self.toggle_mask_ops()
 
         self.update_plot()
