@@ -1,15 +1,22 @@
 import os, datetime, gc, warnings
 import numpy as np
-import skimage.io 
-import matplotlib.pyplot as plt
+import cv2
+import tifffile
 
 from . import plot, transforms
 
 try:
     from PyQt5 import QtGui, QtCore, Qt, QtWidgets
-    GUI = False
-except:
     GUI = True
+except:
+    GUI = False
+
+try:
+    import matplotlib.pyplot as plt
+    MATPLOTLIB = True
+except:
+    MATPLOTLIB = False
+    print('matplotlib not installed')
 
 try:
     from google.cloud import storage
@@ -20,107 +27,180 @@ except:
     SERVER_UPLOAD = False
 
 
+def outlines_to_text(base, outlines):
+    with open(base + '_cp_outlines.txt', 'w') as f:
+        for o in outlines:
+            xy = list(o.flatten())
+            xy_str = ','.join(map(str, xy))
+            f.write(xy_str)
+            f.write('\n')
+
+def imread(filename):
+    ext = os.path.splitext(filename)[-1]
+    if ext== '.tif' or ext=='tiff':
+        img = tifffile.imread(filename)
+        return img
+    else:
+        try:
+            img = cv2.imread(filename, -1)#cv2.LOAD_IMAGE_ANYDEPTH)
+            if img.ndim > 2:
+                img = img[..., [2,1,0]]
+            return img
+        except Exception as e:
+            print('ERROR: could not read file, %s'%e)
+            return None
+
+def imsave(filename, arr):
+    ext = os.path.splitext(filename)[-1]
+    if ext== '.tif' or ext=='tiff':
+        tifffile.imsave(filename, arr)
+    else:
+        cv2.imwrite(filename, arr)
+
 def masks_flows_to_seg(images, masks, flows, diams, file_names, channels=None):
     """ save output of model eval to be loaded in GUI 
 
+    can be list output (run on multiple images) or single output (run on single image)
+
     saved to file_names[k]+'_seg.npy'
     
     Parameters
     -------------
 
-    images: list of 2D or 3D arrays
+    images: (list of) 2D or 3D arrays
         images input into cellpose
 
-    masks: list of 2D arrays, int
+    masks: (list of) 2D arrays, int
         masks output from Cellpose.eval, where 0=NO masks; 1,2,...=mask labels
 
-    flows: list of lists of ND arrays 
+    flows: (list of) list of ND arrays 
         flows output from Cellpose.eval
 
     diams: float array
         diameters used to run Cellpose
 
-    file_names: list of str
+    file_names: (list of) str
         names of files of images
 
     channels: list of int (optional, default None)
         channels used to run Cellpose    
     
     """
-    nimg = len(masks)
+    
     if channels is None:
         channels = [0,0]
-    for n in range(nimg):
-        flowi = []
-        if flows[n][0].ndim==3:
-            flowi.append(flows[n][0][np.newaxis,...])
-        else:
-            flowi.append(flows[n][0])
-        flowi.append((np.clip(transforms.normalize99(flows[n][2]),0,1) * 255).astype(np.uint8)[np.newaxis,...])
-        if flows[n][0].ndim==3:
-            flowi.append(np.zeros(flows[1][0].shape, dtype=np.uint8))
-            flowi[-1] = flowi[-1][np.newaxis,...]
-        else:
-            flowi.append((flows[n][1][0]/10 * 127 + 127).astype(np.uint8))
-        if len(flows[n])>2:
-            flowi.append(flows[n][3])
-            flowi.append(np.concatenate((flows[n][1], flows[n][2][np.newaxis,...]), axis=0))
-        outlines = masks[n] * plot.masks_to_outlines(masks[n])
-        base = os.path.splitext(file_names[n])[0]
-        if images[n].shape[0]<8:
-            np.transpose(images[n], (1,2,0))
-        np.save(base+ '_seg.npy',
-                    {'outlines': outlines.astype(np.uint16),
-                     'masks': masks[n].astype(np.uint16),
-                     'chan_choose': channels,
-                     'img': images[n],
-                     'ismanual': np.zeros(masks[n].max(), np.bool),
-                     'filename': file_names[n],
-                     'flows': flowi,
-                     'est_diam': diams[n]})
+    
+    if isinstance(masks, list):
+        for k, [image, mask, flow, diam, file_name] in enumerate(zip(images, masks, flows, diams, file_names)):
+            channels_img = channels
+            if channels_img is not None and len(channels) > 2:
+                channels_img = channels[k]
+            masks_flows_to_seg(image, mask, flow, diam, file_name, channels_img)
+        return
+
+    flowi = []
+    if flows[0].ndim==3:
+        flowi.append(flows[0][np.newaxis,...])
+    else:
+        flowi.append(flows[0])
+    if flows[0].ndim==3:
+        flowi.append((np.clip(transforms.normalize99(flows[2]),0,1) * 255).astype(np.uint8)[np.newaxis,...])
+        flowi.append(np.zeros(flows[0].shape, dtype=np.uint8))
+        flowi[-1] = flowi[-1][np.newaxis,...]
+    else:
+        flowi.append((np.clip(transforms.normalize99(flows[2]),0,1) * 255).astype(np.uint8))
+        flowi.append((flows[1][0]/10 * 127 + 127).astype(np.uint8))
+    if len(flows)>2:
+        flowi.append(flows[3])
+        flowi.append(np.concatenate((flows[1], flows[2][np.newaxis,...]), axis=0))
+    outlines = masks * plot.masks_to_outlines(masks)
+    base = os.path.splitext(file_names)[0]
+    if images.shape[0]<8:
+        np.transpose(images, (1,2,0))
+    np.save(base+ '_seg.npy',
+                {'outlines': outlines.astype(np.uint16),
+                    'masks': masks.astype(np.uint16),
+                    'chan_choose': channels,
+                    'img': images,
+                    'ismanual': np.zeros(masks.max(), np.bool),
+                    'filename': file_names,
+                    'flows': flowi,
+                    'est_diam': diams})
 
 def save_to_png(images, masks, flows, file_names):
-    """ save masks + nicely plotted segmentation image to png 
+    """ deprecated (runs io.save_masks with png=True) 
+    
+        does not work for 3D images
+    
+    """
+    save_masks(images, masks, flows, file_names, png=True)
 
-    masks[k] for images[k] are saved to file_names[k]+'_cp_masks.png'
+def save_masks(images, masks, flows, file_names, png=True, tif=False):
+    """ save masks + nicely plotted segmentation image to png and/or tiff
 
-    full segmentation figure is saved to file_names[k]+'_cp.png'
+    if png, masks[k] for images[k] are saved to file_names[k]+'_cp_masks.png'
+
+    if tif, masks[k] for images[k] are saved to file_names[k]+'_cp_masks.tif'
+
+    if png and matplotlib installed, full segmentation figure is saved to file_names[k]+'_cp.png'
+
+    only tif option works for 3D data
     
     Parameters
     -------------
 
-    images: list of 2D or 3D arrays
+    images: (list of) 2D, 3D or 4D arrays
         images input into cellpose
 
-    masks: list of 2D arrays, int
+    masks: (list of) 2D arrays, int
         masks output from Cellpose.eval, where 0=NO masks; 1,2,...=mask labels
 
-    flows: list of lists of ND arrays 
+    flows: (list of) list of ND arrays 
         flows output from Cellpose.eval
 
-    file_names: list of str
+    file_names: (list of) str
         names of files of images
     
     """
-    nimg = len(images)
-    for n in range(nimg):
-        img = images[n].copy()
+    
+    if isinstance(masks, list):
+        for image, mask, flow, file_name in zip(images, masks, flows, file_names):
+            save_masks(image, mask, flow, file_name, png=png, tif=tif)
+        return
+    
+    if masks.ndim > 2 and not tif:
+        raise ValueError('cannot save 3D outputs as PNG, use tif option instead')
+
+    base = os.path.splitext(file_names)[0]
+    exts = []
+    if png:
+        exts.append('.png')
+    if tif:
+        exts.append('.tif')
+    
+    # save masks
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore")
+        for ext in exts:
+            imsave(base + '_cp_masks' + ext, masks.astype(np.uint16))
+
+    if png and MATPLOTLIB and not (images.ndim==3 and min(images.shape)>3):
+        img = images.copy()
         if img.ndim<3:
             img = img[:,:,np.newaxis]
         elif img.shape[0]<8:
             np.transpose(img, (1,2,0))
-        base = os.path.splitext(file_names[n])[0]
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore")
-            skimage.io.imsave(base+'_cp_masks.png', masks[n].astype(np.uint16))
-        maski = masks[n]
-        flowi = flows[n][0]
+        
         fig = plt.figure(figsize=(12,3))
         # can save images (set save_dir=None if not)
-        plot.show_segmentation(fig, img, maski, flowi)
-        fig.savefig(base+'_cp.png', dpi=300)
+        plot.show_segmentation(fig, img, masks, flows[0])
+        fig.savefig(base + '_cp_output.png', dpi=300)
         plt.close(fig)
 
+    if masks[0].ndim < 3: 
+        outlines = plot.outlines_list(masks)
+        outlines_to_text(base, outlines)
+
 def save_server(parent=None, filename=None):
     """ Uploads a *_seg.npy file to the bucket.
     
@@ -177,14 +257,15 @@ def _load_image(parent, filename=None):
     manual_file = os.path.splitext(filename)[0]+'_seg.npy'
     if os.path.isfile(manual_file):
         print(manual_file)
-        _load_seg(parent, manual_file, image=skimage.io.imread(filename), image_file=filename)
+        _load_seg(parent, manual_file, image=imread(filename), image_file=filename)
         return
     elif os.path.isfile(os.path.splitext(filename)[0]+'_manual.npy'):
         manual_file = os.path.splitext(filename)[0]+'_manual.npy'
-        _load_seg(parent, manual_file, image=skimage.io.imread(filename), image_file=filename)
+        _load_seg(parent, manual_file, image=imread(filename), image_file=filename)
         return
     try:
-        image = skimage.io.imread(filename)
+        image = imread(filename)
+        image.shape
         parent.loaded = True
     except:
         print('images not compatible')
@@ -198,15 +279,20 @@ def _load_image(parent, filename=None):
         parent.clear_all()
         parent.loaded = True
         parent.enable_buttons()
+        parent.threshslider.setEnabled(False)
+        parent.probslider.setEnabled(False)
+            
 
 def _initialize_images(parent, image, resize, X2):
     """ format image for GUI """
     parent.onechan=False
     if image.ndim > 3:
-        # check tiff is Z x channels x W x H
+        # make tiff Z x channels x W x H
         if image.shape[0]<4:
+            # tiff is channels x Z x W x H
             image = np.transpose(image, (1,0,2,3))
         elif image.shape[-1]<4:
+            # tiff is Z x W x H x channels
             image = np.transpose(image, (0,3,1,2))
         # fill in with blank channels to make 3 channels
         if image.shape[1] < 3:
@@ -301,7 +387,7 @@ def _load_seg(parent, filename=None, image=None, image_file=None):
                     found_image = True
         if found_image:
             try:
-                image = skimage.io.imread(parent.filename)
+                image = imread(parent.filename)
             except:
                 parent.loaded = False
                 found_image = False
@@ -392,13 +478,21 @@ def _load_seg(parent, filename=None, image=None, image_file=None):
         parent.flows = dat['flows']
         try:
             print(parent.flows[0].shape)
+            if parent.NZ==1:
+                parent.threshslider.setEnabled(True)
+                parent.probslider.setEnabled(True)
+            else:
+                parent.threshslider.setEnabled(False)
+                parent.probslider.setEnabled(False)
         except:
             try:
                 if len(parent.flows[0])>0:
                     parent.flows = parent.flows[0]
             except:
-                parent.flows = [[],[],[]]
-
+                parent.flows = [[],[],[],[],[[]]]
+            parent.threshslider.setEnabled(False)
+            parent.probslider.setEnabled(False)
+            
     parent.enable_buttons()
     del dat
     gc.collect()
@@ -410,7 +504,7 @@ def _load_masks(parent, filename=None):
             parent, "Load masks (PNG or TIFF)"
             )
         filename = name[0]
-    masks = skimage.io.imread(filename)
+    masks = imread(filename)
     outlines = None
     if masks.ndim>3:
         # Z x nchannels x Ly x Lx
@@ -464,6 +558,7 @@ def _masks_to_gui(parent, masks, outlines=None):
     if parent.ncells>0:
         parent.toggle_mask_ops()
     parent.ismanual = np.zeros(parent.ncells, np.bool)
+    parent.zdraw = list(-1*np.ones(parent.ncells, np.int16))
     parent.update_plot()
 
 def _save_png(parent):
@@ -472,10 +567,21 @@ def _save_png(parent):
     base = os.path.splitext(filename)[0]
     if parent.NZ==1:
         print('saving 2D masks to png')
-        skimage.io.imsave(base + '_masks.png', parent.cellpix[0])
+        imsave(base + '_cp_masks.png', parent.cellpix[0])
     else:
         print('saving 3D masks to tiff')
-        skimage.io.imsave(base + '_masks.tif', parent.cellpix)
+        imsave(base + '_cp_masks.tif', parent.cellpix)
+
+def _save_outlines(parent):
+    filename = parent.filename
+    base = os.path.splitext(filename)[0]
+    if parent.NZ==1:
+        print('saving 2D outlines to text file, see docs for info to load into ImageJ')    
+        outlines = plot.outlines_list(parent.cellpix[0])
+        outlines_to_text(base, outlines)
+    else:
+        print('ERROR: cannot save 3D outlines')
+    
 
 def _save_sets(parent):
     """ save masks to *_seg.npy """
